/* This file is autogenerated by u_format_access.py from u_format.csv. Do not edit directly. */

/**************************************************************************
 *
 * Copyright 2009 VMware, Inc.
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL VMWARE AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **************************************************************************/

/**
 * @file
 * Pixel format accessor functions.
 *
 * @author Jose Fonseca <jfonseca@vmware.com>
 */

#include "pipe/p_compiler.h"
#include "u_math.h"
#include "u_format_pack.h"

static ubyte srgb_to_linear[256] = {
   0,
   0,
   0,
   0,
   0,
   0,
   0,
   0,
   0,
   0,
   0,
   0,
   0,
   1,
   1,
   1,
   1,
   1,
   1,
   1,
   1,
   1,
   2,
   2,
   2,
   2,
   2,
   2,
   2,
   3,
   3,
   3,
   3,
   3,
   4,
   4,
   4,
   4,
   4,
   5,
   5,
   5,
   5,
   6,
   6,
   6,
   6,
   7,
   7,
   7,
   8,
   8,
   8,
   9,
   9,
   9,
   10,
   10,
   10,
   11,
   11,
   11,
   12,
   12,
   13,
   13,
   13,
   14,
   14,
   15,
   15,
   16,
   16,
   16,
   17,
   17,
   18,
   18,
   19,
   19,
   20,
   20,
   21,
   22,
   22,
   23,
   23,
   24,
   24,
   25,
   26,
   26,
   27,
   27,
   28,
   29,
   29,
   30,
   31,
   31,
   32,
   33,
   33,
   34,
   35,
   36,
   36,
   37,
   38,
   38,
   39,
   40,
   41,
   42,
   42,
   43,
   44,
   45,
   46,
   47,
   47,
   48,
   49,
   50,
   51,
   52,
   53,
   54,
   55,
   55,
   56,
   57,
   58,
   59,
   60,
   61,
   62,
   63,
   64,
   65,
   66,
   67,
   68,
   70,
   71,
   72,
   73,
   74,
   75,
   76,
   77,
   78,
   80,
   81,
   82,
   83,
   84,
   85,
   87,
   88,
   89,
   90,
   92,
   93,
   94,
   95,
   97,
   98,
   99,
   101,
   102,
   103,
   105,
   106,
   107,
   109,
   110,
   112,
   113,
   114,
   116,
   117,
   119,
   120,
   122,
   123,
   125,
   126,
   128,
   129,
   131,
   132,
   134,
   135,
   137,
   139,
   140,
   142,
   144,
   145,
   147,
   148,
   150,
   152,
   153,
   155,
   157,
   159,
   160,
   162,
   164,
   166,
   167,
   169,
   171,
   173,
   175,
   176,
   178,
   180,
   182,
   184,
   186,
   188,
   190,
   192,
   193,
   195,
   197,
   199,
   201,
   203,
   205,
   207,
   209,
   211,
   213,
   215,
   218,
   220,
   222,
   224,
   226,
   228,
   230,
   232,
   235,
   237,
   239,
   241,
   243,
   245,
   248,
   250,
   252,
   255,
};

static ubyte linear_to_srgb[256] = {
   0,
   12,
   21,
   28,
   33,
   38,
   42,
   46,
   49,
   52,
   55,
   58,
   61,
   63,
   66,
   68,
   70,
   73,
   75,
   77,
   79,
   81,
   82,
   84,
   86,
   88,
   89,
   91,
   93,
   94,
   96,
   97,
   99,
   100,
   102,
   103,
   104,
   106,
   107,
   109,
   110,
   111,
   112,
   114,
   115,
   116,
   117,
   118,
   120,
   121,
   122,
   123,
   124,
   125,
   126,
   127,
   129,
   130,
   131,
   132,
   133,
   134,
   135,
   136,
   137,
   138,
   139,
   140,
   141,
   142,
   142,
   143,
   144,
   145,
   146,
   147,
   148,
   149,
   150,
   151,
   151,
   152,
   153,
   154,
   155,
   156,
   157,
   157,
   158,
   159,
   160,
   161,
   161,
   162,
   163,
   164,
   165,
   165,
   166,
   167,
   168,
   168,
   169,
   170,
   171,
   171,
   172,
   173,
   174,
   174,
   175,
   176,
   176,
   177,
   178,
   179,
   179,
   180,
   181,
   181,
   182,
   183,
   183,
   184,
   185,
   185,
   186,
   187,
   187,
   188,
   189,
   189,
   190,
   191,
   191,
   192,
   193,
   193,
   194,
   194,
   195,
   196,
   196,
   197,
   197,
   198,
   199,
   199,
   200,
   201,
   201,
   202,
   202,
   203,
   204,
   204,
   205,
   205,
   206,
   206,
   207,
   208,
   208,
   209,
   209,
   210,
   210,
   211,
   212,
   212,
   213,
   213,
   214,
   214,
   215,
   215,
   216,
   217,
   217,
   218,
   218,
   219,
   219,
   220,
   220,
   221,
   221,
   222,
   222,
   223,
   223,
   224,
   225,
   225,
   226,
   226,
   227,
   227,
   228,
   228,
   229,
   229,
   230,
   230,
   231,
   231,
   232,
   232,
   233,
   233,
   234,
   234,
   235,
   235,
   236,
   236,
   237,
   237,
   237,
   238,
   238,
   239,
   239,
   240,
   240,
   241,
   241,
   242,
   242,
   243,
   243,
   244,
   244,
   245,
   245,
   245,
   246,
   246,
   247,
   247,
   248,
   248,
   249,
   249,
   250,
   250,
   251,
   251,
   251,
   252,
   252,
   253,
   253,
   254,
   254,
   254,
};

static void
util_format_b8g8r8a8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float b = (float)(src_pixel[0] * (1.0f/0xff));
         float g = (float)(src_pixel[1] * (1.0f/0xff));
         float r = (float)(src_pixel[2] * (1.0f/0xff));
         float a = (float)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b8g8r8x8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float b = (float)(src_pixel[0] * (1.0f/0xff));
         float g = (float)(src_pixel[1] * (1.0f/0xff));
         float r = (float)(src_pixel[2] * (1.0f/0xff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_a8r8g8b8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float a = (float)(src_pixel[0] * (1.0f/0xff));
         float r = (float)(src_pixel[1] * (1.0f/0xff));
         float g = (float)(src_pixel[2] * (1.0f/0xff));
         float b = (float)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_x8r8g8b8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[1] * (1.0f/0xff));
         float g = (float)(src_pixel[2] * (1.0f/0xff));
         float b = (float)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_a8b8g8r8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float a = (float)(src_pixel[0] * (1.0f/0xff));
         float b = (float)(src_pixel[1] * (1.0f/0xff));
         float g = (float)(src_pixel[2] * (1.0f/0xff));
         float r = (float)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_x8b8g8r8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float b = (float)(src_pixel[1] * (1.0f/0xff));
         float g = (float)(src_pixel[2] * (1.0f/0xff));
         float r = (float)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b5g5r5a1_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = *src_pixel++;
         float b = (float)((pixel & 0x1f) * (1.0f/0x1f));
         float g = (float)(((pixel >> 5) & 0x1f) * (1.0f/0x1f));
         float r = (float)(((pixel >> 10) & 0x1f) * (1.0f/0x1f));
         float a = (float)((pixel >> 15) * (1.0f/0x1));
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b4g4r4a4_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = *src_pixel++;
         float b = (float)((pixel & 0xf) * (1.0f/0xf));
         float g = (float)(((pixel >> 4) & 0xf) * (1.0f/0xf));
         float r = (float)(((pixel >> 8) & 0xf) * (1.0f/0xf));
         float a = (float)((pixel >> 12) * (1.0f/0xf));
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b5g6r5_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = *src_pixel++;
         float b = (float)((pixel & 0x1f) * (1.0f/0x1f));
         float g = (float)(((pixel >> 5) & 0x3f) * (1.0f/0x3f));
         float r = (float)((pixel >> 11) * (1.0f/0x1f));
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r10g10b10a2_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         float r = (float)((pixel & 0x3ff) * (1.0f/0x3ff));
         float g = (float)(((pixel >> 10) & 0x3ff) * (1.0f/0x3ff));
         float b = (float)(((pixel >> 20) & 0x3ff) * (1.0f/0x3ff));
         float a = (float)((pixel >> 30) * (1.0f/0x3));
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_l8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float rgb = (float)(src_pixel[0] * (1.0f/0xff));
         src_pixel += 1;
         *dst_pixel++ = rgb; /* r */
         *dst_pixel++ = rgb; /* g */
         *dst_pixel++ = rgb; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_a8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float a = (float)(src_pixel[0] * (1.0f/0xff));
         src_pixel += 1;
         *dst_pixel++ = 0; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_i8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float rgba = (float)(src_pixel[0] * (1.0f/0xff));
         src_pixel += 1;
         *dst_pixel++ = rgba; /* r */
         *dst_pixel++ = rgba; /* g */
         *dst_pixel++ = rgba; /* b */
         *dst_pixel++ = rgba; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_l8a8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float rgb = (float)(src_pixel[0] * (1.0f/0xff));
         float a = (float)(src_pixel[1] * (1.0f/0xff));
         src_pixel += 2;
         *dst_pixel++ = rgb; /* r */
         *dst_pixel++ = rgb; /* g */
         *dst_pixel++ = rgb; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_l16_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float rgb = (float)(src_pixel[0] * (1.0f/0xffff));
         src_pixel += 1;
         *dst_pixel++ = rgb; /* r */
         *dst_pixel++ = rgb; /* g */
         *dst_pixel++ = rgb; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z16_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float z = (float)(src_pixel[0] * (1.0f/0xffff));
         src_pixel += 1;
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z32_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float z = (float)(src_pixel[0] * (1.0/0xffffffff));
         src_pixel += 1;
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z32_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float z = src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z24s8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         float z = (float)((pixel & 0xffffff) * (1.0/0xffffff));
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_s8z24_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         float z = (float)((pixel >> 8) * (1.0/0xffffff));
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z24x8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         float z = (float)((pixel & 0xffffff) * (1.0/0xffffff));
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_x8z24_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         float z = (float)((pixel >> 8) * (1.0/0xffffff));
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*8);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64g64_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*16);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64g64b64_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*24);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64g64b64a64_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*32);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         float a = (float)src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*8);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = src_pixel[0];
         float g = src_pixel[1];
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*12);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = src_pixel[0];
         float g = src_pixel[1];
         float b = src_pixel[2];
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32a32_float_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*16);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = src_pixel[0];
         float g = src_pixel[1];
         float b = src_pixel[2];
         float a = src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0/0xffffffff));
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*8);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0/0xffffffff));
         float g = (float)(src_pixel[1] * (1.0/0xffffffff));
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*12);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0/0xffffffff));
         float g = (float)(src_pixel[1] * (1.0/0xffffffff));
         float b = (float)(src_pixel[2] * (1.0/0xffffffff));
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32a32_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*16);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0/0xffffffff));
         float g = (float)(src_pixel[1] * (1.0/0xffffffff));
         float b = (float)(src_pixel[2] * (1.0/0xffffffff));
         float a = (float)(src_pixel[3] * (1.0/0xffffffff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*8);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*12);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32a32_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*16);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         float a = (float)src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xffff));
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xffff));
         float g = (float)(src_pixel[1] * (1.0f/0xffff));
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*6);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xffff));
         float g = (float)(src_pixel[1] * (1.0f/0xffff));
         float b = (float)(src_pixel[2] * (1.0f/0xffff));
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16a16_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*8);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xffff));
         float g = (float)(src_pixel[1] * (1.0f/0xffff));
         float b = (float)(src_pixel[2] * (1.0f/0xffff));
         float a = (float)(src_pixel[3] * (1.0f/0xffff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*6);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16a16_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*8);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         float a = (float)src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xff));
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xff));
         float g = (float)(src_pixel[1] * (1.0f/0xff));
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*3);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xff));
         float g = (float)(src_pixel[1] * (1.0f/0xff));
         float b = (float)(src_pixel[2] * (1.0f/0xff));
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8a8_unorm_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)(src_pixel[0] * (1.0f/0xff));
         float g = (float)(src_pixel[1] * (1.0f/0xff));
         float b = (float)(src_pixel[2] * (1.0f/0xff));
         float a = (float)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*2);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*3);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 1; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8a8_uscaled_read_4f(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   float *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      float *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         float r = (float)src_pixel[0];
         float g = (float)src_pixel[1];
         float b = (float)src_pixel[2];
         float a = (float)src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

void
util_format_read_4f(enum pipe_format format, float *dst, unsigned dst_stride, const void *src, unsigned src_stride, unsigned x, unsigned y, unsigned w, unsigned h)
{
   void (*func)(float *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h);
   switch(format) {
   case PIPE_FORMAT_B8G8R8A8_UNORM:
      func = &util_format_b8g8r8a8_unorm_read_4f;
      break;
   case PIPE_FORMAT_B8G8R8X8_UNORM:
      func = &util_format_b8g8r8x8_unorm_read_4f;
      break;
   case PIPE_FORMAT_A8R8G8B8_UNORM:
      func = &util_format_a8r8g8b8_unorm_read_4f;
      break;
   case PIPE_FORMAT_X8R8G8B8_UNORM:
      func = &util_format_x8r8g8b8_unorm_read_4f;
      break;
   case PIPE_FORMAT_A8B8G8R8_UNORM:
      func = &util_format_a8b8g8r8_unorm_read_4f;
      break;
   case PIPE_FORMAT_X8B8G8R8_UNORM:
      func = &util_format_x8b8g8r8_unorm_read_4f;
      break;
   case PIPE_FORMAT_B5G5R5A1_UNORM:
      func = &util_format_b5g5r5a1_unorm_read_4f;
      break;
   case PIPE_FORMAT_B4G4R4A4_UNORM:
      func = &util_format_b4g4r4a4_unorm_read_4f;
      break;
   case PIPE_FORMAT_B5G6R5_UNORM:
      func = &util_format_b5g6r5_unorm_read_4f;
      break;
   case PIPE_FORMAT_R10G10B10A2_UNORM:
      func = &util_format_r10g10b10a2_unorm_read_4f;
      break;
   case PIPE_FORMAT_L8_UNORM:
      func = &util_format_l8_unorm_read_4f;
      break;
   case PIPE_FORMAT_A8_UNORM:
      func = &util_format_a8_unorm_read_4f;
      break;
   case PIPE_FORMAT_I8_UNORM:
      func = &util_format_i8_unorm_read_4f;
      break;
   case PIPE_FORMAT_L8A8_UNORM:
      func = &util_format_l8a8_unorm_read_4f;
      break;
   case PIPE_FORMAT_L16_UNORM:
      func = &util_format_l16_unorm_read_4f;
      break;
   case PIPE_FORMAT_Z16_UNORM:
      func = &util_format_z16_unorm_read_4f;
      break;
   case PIPE_FORMAT_Z32_UNORM:
      func = &util_format_z32_unorm_read_4f;
      break;
   case PIPE_FORMAT_Z32_FLOAT:
      func = &util_format_z32_float_read_4f;
      break;
   case PIPE_FORMAT_Z24S8_UNORM:
      func = &util_format_z24s8_unorm_read_4f;
      break;
   case PIPE_FORMAT_S8Z24_UNORM:
      func = &util_format_s8z24_unorm_read_4f;
      break;
   case PIPE_FORMAT_Z24X8_UNORM:
      func = &util_format_z24x8_unorm_read_4f;
      break;
   case PIPE_FORMAT_X8Z24_UNORM:
      func = &util_format_x8z24_unorm_read_4f;
      break;
   case PIPE_FORMAT_R64_FLOAT:
      func = &util_format_r64_float_read_4f;
      break;
   case PIPE_FORMAT_R64G64_FLOAT:
      func = &util_format_r64g64_float_read_4f;
      break;
   case PIPE_FORMAT_R64G64B64_FLOAT:
      func = &util_format_r64g64b64_float_read_4f;
      break;
   case PIPE_FORMAT_R64G64B64A64_FLOAT:
      func = &util_format_r64g64b64a64_float_read_4f;
      break;
   case PIPE_FORMAT_R32_FLOAT:
      func = &util_format_r32_float_read_4f;
      break;
   case PIPE_FORMAT_R32G32_FLOAT:
      func = &util_format_r32g32_float_read_4f;
      break;
   case PIPE_FORMAT_R32G32B32_FLOAT:
      func = &util_format_r32g32b32_float_read_4f;
      break;
   case PIPE_FORMAT_R32G32B32A32_FLOAT:
      func = &util_format_r32g32b32a32_float_read_4f;
      break;
   case PIPE_FORMAT_R32_UNORM:
      func = &util_format_r32_unorm_read_4f;
      break;
   case PIPE_FORMAT_R32G32_UNORM:
      func = &util_format_r32g32_unorm_read_4f;
      break;
   case PIPE_FORMAT_R32G32B32_UNORM:
      func = &util_format_r32g32b32_unorm_read_4f;
      break;
   case PIPE_FORMAT_R32G32B32A32_UNORM:
      func = &util_format_r32g32b32a32_unorm_read_4f;
      break;
   case PIPE_FORMAT_R32_USCALED:
      func = &util_format_r32_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R32G32_USCALED:
      func = &util_format_r32g32_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R32G32B32_USCALED:
      func = &util_format_r32g32b32_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R32G32B32A32_USCALED:
      func = &util_format_r32g32b32a32_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R16_UNORM:
      func = &util_format_r16_unorm_read_4f;
      break;
   case PIPE_FORMAT_R16G16_UNORM:
      func = &util_format_r16g16_unorm_read_4f;
      break;
   case PIPE_FORMAT_R16G16B16_UNORM:
      func = &util_format_r16g16b16_unorm_read_4f;
      break;
   case PIPE_FORMAT_R16G16B16A16_UNORM:
      func = &util_format_r16g16b16a16_unorm_read_4f;
      break;
   case PIPE_FORMAT_R16_USCALED:
      func = &util_format_r16_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R16G16_USCALED:
      func = &util_format_r16g16_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R16G16B16_USCALED:
      func = &util_format_r16g16b16_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R16G16B16A16_USCALED:
      func = &util_format_r16g16b16a16_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R8_UNORM:
      func = &util_format_r8_unorm_read_4f;
      break;
   case PIPE_FORMAT_R8G8_UNORM:
      func = &util_format_r8g8_unorm_read_4f;
      break;
   case PIPE_FORMAT_R8G8B8_UNORM:
      func = &util_format_r8g8b8_unorm_read_4f;
      break;
   case PIPE_FORMAT_R8G8B8A8_UNORM:
      func = &util_format_r8g8b8a8_unorm_read_4f;
      break;
   case PIPE_FORMAT_R8_USCALED:
      func = &util_format_r8_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R8G8_USCALED:
      func = &util_format_r8g8_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R8G8B8_USCALED:
      func = &util_format_r8g8b8_uscaled_read_4f;
      break;
   case PIPE_FORMAT_R8G8B8A8_USCALED:
      func = &util_format_r8g8b8a8_uscaled_read_4f;
      break;
   default:
      debug_printf("unsupported format\n");
      return;
   }
   func(dst, dst_stride, (const uint8_t *)src, src_stride, x, y, w, h);
}

static void
util_format_b8g8r8a8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b8g8r8x8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_a8r8g8b8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_x8r8g8b8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_a8b8g8r8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_x8b8g8r8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b5g5r5a1_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = 0;
         pixel |= (uint16_t)(CLAMP(src_pixel[2], 0, 1) * 0x1f);
         pixel |= ((uint16_t)(CLAMP(src_pixel[1], 0, 1) * 0x1f) << 5);
         pixel |= ((uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0x1f) << 10);
         pixel |= ((uint16_t)(CLAMP(src_pixel[3], 0, 1) * 0x1) << 15);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b4g4r4a4_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = 0;
         pixel |= (uint16_t)(CLAMP(src_pixel[2], 0, 1) * 0xf);
         pixel |= ((uint16_t)(CLAMP(src_pixel[1], 0, 1) * 0xf) << 4);
         pixel |= ((uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0xf) << 8);
         pixel |= ((uint16_t)(CLAMP(src_pixel[3], 0, 1) * 0xf) << 12);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b5g6r5_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = 0;
         pixel |= (uint16_t)(CLAMP(src_pixel[2], 0, 1) * 0x1f);
         pixel |= ((uint16_t)(CLAMP(src_pixel[1], 0, 1) * 0x3f) << 5);
         pixel |= ((uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0x1f) << 11);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r10g10b10a2_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= (uint32_t)(CLAMP(src_pixel[0], 0, 1) * 0x3ff);
         pixel |= ((uint32_t)(CLAMP(src_pixel[1], 0, 1) * 0x3ff) << 10);
         pixel |= ((uint32_t)(CLAMP(src_pixel[2], 0, 1) * 0x3ff) << 20);
         pixel |= ((uint32_t)(CLAMP(src_pixel[3], 0, 1) * 0x3) << 30);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_l8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_a8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_i8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_l8a8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_l16_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[2], 0, 1) * 0xffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z16_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0xffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z32_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z32_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z24s8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= (uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffff);
         pixel |= ((uint32_t)(CLAMP(src_pixel[1], 0, 1) * 0xff) << 24);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_s8z24_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= (uint32_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         pixel |= ((uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffff) << 8);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z24x8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= (uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffff);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_x8z24_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= ((uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffff) << 8);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*8);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64g64_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*16);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)src_pixel[0];
         *dst_pixel++ = (double)src_pixel[1];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64g64b64_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*24);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)src_pixel[0];
         *dst_pixel++ = (double)src_pixel[1];
         *dst_pixel++ = (double)src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64g64b64a64_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*32);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)src_pixel[0];
         *dst_pixel++ = (double)src_pixel[1];
         *dst_pixel++ = (double)src_pixel[2];
         *dst_pixel++ = (double)src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*8);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*12);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32a32_float_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*16);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[2];
         *dst_pixel++ = src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*8);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffffff);
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[1], 0, 1) * (double)0xffffffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*12);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffffff);
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[1], 0, 1) * (double)0xffffffff);
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[2], 0, 1) * (double)0xffffffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32a32_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*16);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[0], 0, 1) * (double)0xffffffff);
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[1], 0, 1) * (double)0xffffffff);
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[2], 0, 1) * (double)0xffffffff);
         *dst_pixel++ = (uint32_t)(CLAMP(src_pixel[3], 0, 1) * (double)0xffffffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*8);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)src_pixel[0];
         *dst_pixel++ = (uint32_t)src_pixel[1];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*12);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)src_pixel[0];
         *dst_pixel++ = (uint32_t)src_pixel[1];
         *dst_pixel++ = (uint32_t)src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32a32_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*16);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)src_pixel[0];
         *dst_pixel++ = (uint32_t)src_pixel[1];
         *dst_pixel++ = (uint32_t)src_pixel[2];
         *dst_pixel++ = (uint32_t)src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0xffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0xffff);
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[1], 0, 1) * 0xffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*6);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0xffff);
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[1], 0, 1) * 0xffff);
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[2], 0, 1) * 0xffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16a16_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*8);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[0], 0, 1) * 0xffff);
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[1], 0, 1) * 0xffff);
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[2], 0, 1) * 0xffff);
         *dst_pixel++ = (uint16_t)(CLAMP(src_pixel[3], 0, 1) * 0xffff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)src_pixel[0];
         *dst_pixel++ = (uint16_t)src_pixel[1];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*6);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)src_pixel[0];
         *dst_pixel++ = (uint16_t)src_pixel[1];
         *dst_pixel++ = (uint16_t)src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16a16_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*8);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)src_pixel[0];
         *dst_pixel++ = (uint16_t)src_pixel[1];
         *dst_pixel++ = (uint16_t)src_pixel[2];
         *dst_pixel++ = (uint16_t)src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*3);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8a8_unorm_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         *dst_pixel++ = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*2);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)src_pixel[0];
         *dst_pixel++ = (uint8_t)src_pixel[1];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*3);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)src_pixel[0];
         *dst_pixel++ = (uint8_t)src_pixel[1];
         *dst_pixel++ = (uint8_t)src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8a8_uscaled_write_4f(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const float *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const float *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)src_pixel[0];
         *dst_pixel++ = (uint8_t)src_pixel[1];
         *dst_pixel++ = (uint8_t)src_pixel[2];
         *dst_pixel++ = (uint8_t)src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

void
util_format_write_4f(enum pipe_format format, const float *src, unsigned src_stride, void *dst, unsigned dst_stride, unsigned x, unsigned y, unsigned w, unsigned h)
{
   void (*func)(const float *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h);
   switch(format) {
   case PIPE_FORMAT_B8G8R8A8_UNORM:
      func = &util_format_b8g8r8a8_unorm_write_4f;
      break;
   case PIPE_FORMAT_B8G8R8X8_UNORM:
      func = &util_format_b8g8r8x8_unorm_write_4f;
      break;
   case PIPE_FORMAT_A8R8G8B8_UNORM:
      func = &util_format_a8r8g8b8_unorm_write_4f;
      break;
   case PIPE_FORMAT_X8R8G8B8_UNORM:
      func = &util_format_x8r8g8b8_unorm_write_4f;
      break;
   case PIPE_FORMAT_A8B8G8R8_UNORM:
      func = &util_format_a8b8g8r8_unorm_write_4f;
      break;
   case PIPE_FORMAT_X8B8G8R8_UNORM:
      func = &util_format_x8b8g8r8_unorm_write_4f;
      break;
   case PIPE_FORMAT_B5G5R5A1_UNORM:
      func = &util_format_b5g5r5a1_unorm_write_4f;
      break;
   case PIPE_FORMAT_B4G4R4A4_UNORM:
      func = &util_format_b4g4r4a4_unorm_write_4f;
      break;
   case PIPE_FORMAT_B5G6R5_UNORM:
      func = &util_format_b5g6r5_unorm_write_4f;
      break;
   case PIPE_FORMAT_R10G10B10A2_UNORM:
      func = &util_format_r10g10b10a2_unorm_write_4f;
      break;
   case PIPE_FORMAT_L8_UNORM:
      func = &util_format_l8_unorm_write_4f;
      break;
   case PIPE_FORMAT_A8_UNORM:
      func = &util_format_a8_unorm_write_4f;
      break;
   case PIPE_FORMAT_I8_UNORM:
      func = &util_format_i8_unorm_write_4f;
      break;
   case PIPE_FORMAT_L8A8_UNORM:
      func = &util_format_l8a8_unorm_write_4f;
      break;
   case PIPE_FORMAT_L16_UNORM:
      func = &util_format_l16_unorm_write_4f;
      break;
   case PIPE_FORMAT_Z16_UNORM:
      func = &util_format_z16_unorm_write_4f;
      break;
   case PIPE_FORMAT_Z32_UNORM:
      func = &util_format_z32_unorm_write_4f;
      break;
   case PIPE_FORMAT_Z32_FLOAT:
      func = &util_format_z32_float_write_4f;
      break;
   case PIPE_FORMAT_Z24S8_UNORM:
      func = &util_format_z24s8_unorm_write_4f;
      break;
   case PIPE_FORMAT_S8Z24_UNORM:
      func = &util_format_s8z24_unorm_write_4f;
      break;
   case PIPE_FORMAT_Z24X8_UNORM:
      func = &util_format_z24x8_unorm_write_4f;
      break;
   case PIPE_FORMAT_X8Z24_UNORM:
      func = &util_format_x8z24_unorm_write_4f;
      break;
   case PIPE_FORMAT_R64_FLOAT:
      func = &util_format_r64_float_write_4f;
      break;
   case PIPE_FORMAT_R64G64_FLOAT:
      func = &util_format_r64g64_float_write_4f;
      break;
   case PIPE_FORMAT_R64G64B64_FLOAT:
      func = &util_format_r64g64b64_float_write_4f;
      break;
   case PIPE_FORMAT_R64G64B64A64_FLOAT:
      func = &util_format_r64g64b64a64_float_write_4f;
      break;
   case PIPE_FORMAT_R32_FLOAT:
      func = &util_format_r32_float_write_4f;
      break;
   case PIPE_FORMAT_R32G32_FLOAT:
      func = &util_format_r32g32_float_write_4f;
      break;
   case PIPE_FORMAT_R32G32B32_FLOAT:
      func = &util_format_r32g32b32_float_write_4f;
      break;
   case PIPE_FORMAT_R32G32B32A32_FLOAT:
      func = &util_format_r32g32b32a32_float_write_4f;
      break;
   case PIPE_FORMAT_R32_UNORM:
      func = &util_format_r32_unorm_write_4f;
      break;
   case PIPE_FORMAT_R32G32_UNORM:
      func = &util_format_r32g32_unorm_write_4f;
      break;
   case PIPE_FORMAT_R32G32B32_UNORM:
      func = &util_format_r32g32b32_unorm_write_4f;
      break;
   case PIPE_FORMAT_R32G32B32A32_UNORM:
      func = &util_format_r32g32b32a32_unorm_write_4f;
      break;
   case PIPE_FORMAT_R32_USCALED:
      func = &util_format_r32_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R32G32_USCALED:
      func = &util_format_r32g32_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R32G32B32_USCALED:
      func = &util_format_r32g32b32_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R32G32B32A32_USCALED:
      func = &util_format_r32g32b32a32_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R16_UNORM:
      func = &util_format_r16_unorm_write_4f;
      break;
   case PIPE_FORMAT_R16G16_UNORM:
      func = &util_format_r16g16_unorm_write_4f;
      break;
   case PIPE_FORMAT_R16G16B16_UNORM:
      func = &util_format_r16g16b16_unorm_write_4f;
      break;
   case PIPE_FORMAT_R16G16B16A16_UNORM:
      func = &util_format_r16g16b16a16_unorm_write_4f;
      break;
   case PIPE_FORMAT_R16_USCALED:
      func = &util_format_r16_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R16G16_USCALED:
      func = &util_format_r16g16_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R16G16B16_USCALED:
      func = &util_format_r16g16b16_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R16G16B16A16_USCALED:
      func = &util_format_r16g16b16a16_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R8_UNORM:
      func = &util_format_r8_unorm_write_4f;
      break;
   case PIPE_FORMAT_R8G8_UNORM:
      func = &util_format_r8g8_unorm_write_4f;
      break;
   case PIPE_FORMAT_R8G8B8_UNORM:
      func = &util_format_r8g8b8_unorm_write_4f;
      break;
   case PIPE_FORMAT_R8G8B8A8_UNORM:
      func = &util_format_r8g8b8a8_unorm_write_4f;
      break;
   case PIPE_FORMAT_R8_USCALED:
      func = &util_format_r8_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R8G8_USCALED:
      func = &util_format_r8g8_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R8G8B8_USCALED:
      func = &util_format_r8g8b8_uscaled_write_4f;
      break;
   case PIPE_FORMAT_R8G8B8A8_USCALED:
      func = &util_format_r8g8b8a8_uscaled_write_4f;
      break;
   default:
      debug_printf("unsupported format\n");
      return;
   }
   func(src, src_stride, (uint8_t *)dst, dst_stride, x, y, w, h);
}

static void
util_format_b8g8r8a8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t b = src_pixel[0];
         uint8_t g = src_pixel[1];
         uint8_t r = src_pixel[2];
         uint8_t a = src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b8g8r8x8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t b = src_pixel[0];
         uint8_t g = src_pixel[1];
         uint8_t r = src_pixel[2];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_a8r8g8b8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t a = src_pixel[0];
         uint8_t r = src_pixel[1];
         uint8_t g = src_pixel[2];
         uint8_t b = src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_x8r8g8b8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = src_pixel[1];
         uint8_t g = src_pixel[2];
         uint8_t b = src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_a8b8g8r8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t a = src_pixel[0];
         uint8_t b = src_pixel[1];
         uint8_t g = src_pixel[2];
         uint8_t r = src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_x8b8g8r8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t b = src_pixel[1];
         uint8_t g = src_pixel[2];
         uint8_t r = src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b5g5r5a1_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = *src_pixel++;
         uint8_t b = (uint8_t)((uint32_t)(pixel & 0x1f) * 0xff / 0x1f);
         uint8_t g = (uint8_t)((uint32_t)((pixel >> 5) & 0x1f) * 0xff / 0x1f);
         uint8_t r = (uint8_t)((uint32_t)((pixel >> 10) & 0x1f) * 0xff / 0x1f);
         uint8_t a = (uint8_t)((uint32_t)(pixel >> 15) * 0xff / 0x1);
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b4g4r4a4_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = *src_pixel++;
         uint8_t b = (uint8_t)((uint32_t)(pixel & 0xf) * 0xff / 0xf);
         uint8_t g = (uint8_t)((uint32_t)((pixel >> 4) & 0xf) * 0xff / 0xf);
         uint8_t r = (uint8_t)((uint32_t)((pixel >> 8) & 0xf) * 0xff / 0xf);
         uint8_t a = (uint8_t)((uint32_t)(pixel >> 12) * 0xff / 0xf);
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_b5g6r5_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = *src_pixel++;
         uint8_t b = (uint8_t)((uint32_t)(pixel & 0x1f) * 0xff / 0x1f);
         uint8_t g = (uint8_t)((uint32_t)((pixel >> 5) & 0x3f) * 0xff / 0x3f);
         uint8_t r = (uint8_t)((uint32_t)(pixel >> 11) * 0xff / 0x1f);
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r10g10b10a2_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         uint8_t r = (uint8_t)((pixel & 0x3ff) >> 2);
         uint8_t g = (uint8_t)(((pixel >> 10) & 0x3ff) >> 2);
         uint8_t b = (uint8_t)(((pixel >> 20) & 0x3ff) >> 2);
         uint8_t a = (uint8_t)((uint32_t)(pixel >> 30) * 0xff / 0x3);
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_l8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t rgb = src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = rgb; /* r */
         *dst_pixel++ = rgb; /* g */
         *dst_pixel++ = rgb; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_a8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t a = src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = 0; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_i8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t rgba = src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = rgba; /* r */
         *dst_pixel++ = rgba; /* g */
         *dst_pixel++ = rgba; /* b */
         *dst_pixel++ = rgba; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_l8a8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t rgb = src_pixel[0];
         uint8_t a = src_pixel[1];
         src_pixel += 2;
         *dst_pixel++ = rgb; /* r */
         *dst_pixel++ = rgb; /* g */
         *dst_pixel++ = rgb; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_l16_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t rgb = (uint8_t)(src_pixel[0] >> 8);
         src_pixel += 1;
         *dst_pixel++ = rgb; /* r */
         *dst_pixel++ = rgb; /* g */
         *dst_pixel++ = rgb; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z16_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t z = (uint8_t)(src_pixel[0] >> 8);
         src_pixel += 1;
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z32_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t z = (uint8_t)(src_pixel[0] >> 24);
         src_pixel += 1;
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z32_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t z = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         src_pixel += 1;
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z24s8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         uint8_t z = (uint8_t)((pixel & 0xffffff) >> 16);
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_s8z24_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         uint8_t z = (uint8_t)((pixel >> 8) >> 16);
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_z24x8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         uint8_t z = (uint8_t)((pixel & 0xffffff) >> 16);
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_x8z24_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = *src_pixel++;
         uint8_t z = (uint8_t)((pixel >> 8) >> 16);
         *dst_pixel++ = z; /* r */
         *dst_pixel++ = z; /* g */
         *dst_pixel++ = z; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*8);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64g64_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*16);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         uint8_t g = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64g64b64_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*24);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         uint8_t g = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         uint8_t b = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r64g64b64a64_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const double *src_pixel = (const double *)(src_row + x0*32);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         uint8_t g = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         uint8_t b = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         uint8_t a = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*8);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         uint8_t g = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*12);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         uint8_t g = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         uint8_t b = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32a32_float_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const float *src_pixel = (const float *)(src_row + x0*16);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(CLAMP(src_pixel[0], 0, 1) * 0xff);
         uint8_t g = (uint8_t)(CLAMP(src_pixel[1], 0, 1) * 0xff);
         uint8_t b = (uint8_t)(CLAMP(src_pixel[2], 0, 1) * 0xff);
         uint8_t a = (uint8_t)(CLAMP(src_pixel[3], 0, 1) * 0xff);
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 24);
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*8);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 24);
         uint8_t g = (uint8_t)(src_pixel[1] >> 24);
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*12);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 24);
         uint8_t g = (uint8_t)(src_pixel[1] >> 24);
         uint8_t b = (uint8_t)(src_pixel[2] >> 24);
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32a32_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*16);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 24);
         uint8_t g = (uint8_t)(src_pixel[1] >> 24);
         uint8_t b = (uint8_t)(src_pixel[2] >> 24);
         uint8_t a = (uint8_t)(src_pixel[3] >> 24);
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint64_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*8);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint64_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint64_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*12);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint64_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint64_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         uint8_t b = (uint8_t)((uint64_t)MIN2(src_pixel[2], 1) * 0xff / 0x1);
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r32g32b32a32_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint32_t *src_pixel = (const uint32_t *)(src_row + x0*16);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint64_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint64_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         uint8_t b = (uint8_t)((uint64_t)MIN2(src_pixel[2], 1) * 0xff / 0x1);
         uint8_t a = (uint8_t)((uint64_t)MIN2(src_pixel[3], 1) * 0xff / 0x1);
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 8);
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 8);
         uint8_t g = (uint8_t)(src_pixel[1] >> 8);
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*6);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 8);
         uint8_t g = (uint8_t)(src_pixel[1] >> 8);
         uint8_t b = (uint8_t)(src_pixel[2] >> 8);
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16a16_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*8);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)(src_pixel[0] >> 8);
         uint8_t g = (uint8_t)(src_pixel[1] >> 8);
         uint8_t b = (uint8_t)(src_pixel[2] >> 8);
         uint8_t a = (uint8_t)(src_pixel[3] >> 8);
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint32_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*6);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint32_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         uint8_t b = (uint8_t)((uint32_t)MIN2(src_pixel[2], 1) * 0xff / 0x1);
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r16g16b16a16_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint16_t *src_pixel = (const uint16_t *)(src_row + x0*8);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint32_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         uint8_t b = (uint8_t)((uint32_t)MIN2(src_pixel[2], 1) * 0xff / 0x1);
         uint8_t a = (uint8_t)((uint32_t)MIN2(src_pixel[3], 1) * 0xff / 0x1);
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = src_pixel[0];
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = src_pixel[0];
         uint8_t g = src_pixel[1];
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*3);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = src_pixel[0];
         uint8_t g = src_pixel[1];
         uint8_t b = src_pixel[2];
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8a8_unorm_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = src_pixel[0];
         uint8_t g = src_pixel[1];
         uint8_t b = src_pixel[2];
         uint8_t a = src_pixel[3];
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*1);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         src_pixel += 1;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = 0; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*2);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint32_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         src_pixel += 2;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = 0; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*3);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint32_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         uint8_t b = (uint8_t)((uint32_t)MIN2(src_pixel[2], 1) * 0xff / 0x1);
         src_pixel += 3;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = 255; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

static void
util_format_r8g8b8a8_uscaled_read_4ub(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   const uint8_t *src_row = src + y0*src_stride;
   uint8_t *dst_row = dst;
   for (y = 0; y < h; ++y) {
      const uint8_t *src_pixel = (const uint8_t *)(src_row + x0*4);
      uint8_t *dst_pixel = dst_row;
      for (x = 0; x < w; ++x) {
         uint8_t r = (uint8_t)((uint32_t)MIN2(src_pixel[0], 1) * 0xff / 0x1);
         uint8_t g = (uint8_t)((uint32_t)MIN2(src_pixel[1], 1) * 0xff / 0x1);
         uint8_t b = (uint8_t)((uint32_t)MIN2(src_pixel[2], 1) * 0xff / 0x1);
         uint8_t a = (uint8_t)((uint32_t)MIN2(src_pixel[3], 1) * 0xff / 0x1);
         src_pixel += 4;
         *dst_pixel++ = r; /* r */
         *dst_pixel++ = g; /* g */
         *dst_pixel++ = b; /* b */
         *dst_pixel++ = a; /* a */
      }
      src_row += src_stride;
      dst_row += dst_stride/sizeof(*dst_row);
   }
}

void
util_format_read_4ub(enum pipe_format format, uint8_t *dst, unsigned dst_stride, const void *src, unsigned src_stride, unsigned x, unsigned y, unsigned w, unsigned h)
{
   void (*func)(uint8_t *dst, unsigned dst_stride, const uint8_t *src, unsigned src_stride, unsigned x0, unsigned y0, unsigned w, unsigned h);
   switch(format) {
   case PIPE_FORMAT_B8G8R8A8_UNORM:
      func = &util_format_b8g8r8a8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_B8G8R8X8_UNORM:
      func = &util_format_b8g8r8x8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_A8R8G8B8_UNORM:
      func = &util_format_a8r8g8b8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_X8R8G8B8_UNORM:
      func = &util_format_x8r8g8b8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_A8B8G8R8_UNORM:
      func = &util_format_a8b8g8r8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_X8B8G8R8_UNORM:
      func = &util_format_x8b8g8r8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_B5G5R5A1_UNORM:
      func = &util_format_b5g5r5a1_unorm_read_4ub;
      break;
   case PIPE_FORMAT_B4G4R4A4_UNORM:
      func = &util_format_b4g4r4a4_unorm_read_4ub;
      break;
   case PIPE_FORMAT_B5G6R5_UNORM:
      func = &util_format_b5g6r5_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R10G10B10A2_UNORM:
      func = &util_format_r10g10b10a2_unorm_read_4ub;
      break;
   case PIPE_FORMAT_L8_UNORM:
      func = &util_format_l8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_A8_UNORM:
      func = &util_format_a8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_I8_UNORM:
      func = &util_format_i8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_L8A8_UNORM:
      func = &util_format_l8a8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_L16_UNORM:
      func = &util_format_l16_unorm_read_4ub;
      break;
   case PIPE_FORMAT_Z16_UNORM:
      func = &util_format_z16_unorm_read_4ub;
      break;
   case PIPE_FORMAT_Z32_UNORM:
      func = &util_format_z32_unorm_read_4ub;
      break;
   case PIPE_FORMAT_Z32_FLOAT:
      func = &util_format_z32_float_read_4ub;
      break;
   case PIPE_FORMAT_Z24S8_UNORM:
      func = &util_format_z24s8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_S8Z24_UNORM:
      func = &util_format_s8z24_unorm_read_4ub;
      break;
   case PIPE_FORMAT_Z24X8_UNORM:
      func = &util_format_z24x8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_X8Z24_UNORM:
      func = &util_format_x8z24_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R64_FLOAT:
      func = &util_format_r64_float_read_4ub;
      break;
   case PIPE_FORMAT_R64G64_FLOAT:
      func = &util_format_r64g64_float_read_4ub;
      break;
   case PIPE_FORMAT_R64G64B64_FLOAT:
      func = &util_format_r64g64b64_float_read_4ub;
      break;
   case PIPE_FORMAT_R64G64B64A64_FLOAT:
      func = &util_format_r64g64b64a64_float_read_4ub;
      break;
   case PIPE_FORMAT_R32_FLOAT:
      func = &util_format_r32_float_read_4ub;
      break;
   case PIPE_FORMAT_R32G32_FLOAT:
      func = &util_format_r32g32_float_read_4ub;
      break;
   case PIPE_FORMAT_R32G32B32_FLOAT:
      func = &util_format_r32g32b32_float_read_4ub;
      break;
   case PIPE_FORMAT_R32G32B32A32_FLOAT:
      func = &util_format_r32g32b32a32_float_read_4ub;
      break;
   case PIPE_FORMAT_R32_UNORM:
      func = &util_format_r32_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R32G32_UNORM:
      func = &util_format_r32g32_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R32G32B32_UNORM:
      func = &util_format_r32g32b32_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R32G32B32A32_UNORM:
      func = &util_format_r32g32b32a32_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R32_USCALED:
      func = &util_format_r32_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R32G32_USCALED:
      func = &util_format_r32g32_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R32G32B32_USCALED:
      func = &util_format_r32g32b32_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R32G32B32A32_USCALED:
      func = &util_format_r32g32b32a32_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R16_UNORM:
      func = &util_format_r16_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R16G16_UNORM:
      func = &util_format_r16g16_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R16G16B16_UNORM:
      func = &util_format_r16g16b16_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R16G16B16A16_UNORM:
      func = &util_format_r16g16b16a16_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R16_USCALED:
      func = &util_format_r16_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R16G16_USCALED:
      func = &util_format_r16g16_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R16G16B16_USCALED:
      func = &util_format_r16g16b16_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R16G16B16A16_USCALED:
      func = &util_format_r16g16b16a16_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R8_UNORM:
      func = &util_format_r8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R8G8_UNORM:
      func = &util_format_r8g8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R8G8B8_UNORM:
      func = &util_format_r8g8b8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R8G8B8A8_UNORM:
      func = &util_format_r8g8b8a8_unorm_read_4ub;
      break;
   case PIPE_FORMAT_R8_USCALED:
      func = &util_format_r8_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R8G8_USCALED:
      func = &util_format_r8g8_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R8G8B8_USCALED:
      func = &util_format_r8g8b8_uscaled_read_4ub;
      break;
   case PIPE_FORMAT_R8G8B8A8_USCALED:
      func = &util_format_r8g8b8a8_uscaled_read_4ub;
      break;
   default:
      debug_printf("unsupported format\n");
      return;
   }
   func(dst, dst_stride, (const uint8_t *)src, src_stride, x, y, w, h);
}

static void
util_format_b8g8r8a8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[2];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b8g8r8x8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[2];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_a8r8g8b8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[3];
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_x8r8g8b8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_a8b8g8r8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[3];
         *dst_pixel++ = src_pixel[2];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_x8b8g8r8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[2];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b5g5r5a1_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = 0;
         pixel |= (uint16_t)(src_pixel[2] >> 3);
         pixel |= ((uint16_t)(src_pixel[1] >> 3) << 5);
         pixel |= ((uint16_t)(src_pixel[0] >> 3) << 10);
         pixel |= ((uint16_t)(src_pixel[3] >> 7) << 15);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b4g4r4a4_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = 0;
         pixel |= (uint16_t)(src_pixel[2] >> 4);
         pixel |= ((uint16_t)(src_pixel[1] >> 4) << 4);
         pixel |= ((uint16_t)(src_pixel[0] >> 4) << 8);
         pixel |= ((uint16_t)(src_pixel[3] >> 4) << 12);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_b5g6r5_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint16_t pixel = 0;
         pixel |= (uint16_t)(src_pixel[2] >> 3);
         pixel |= ((uint16_t)(src_pixel[1] >> 2) << 5);
         pixel |= ((uint16_t)(src_pixel[0] >> 3) << 11);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r10g10b10a2_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= (uint32_t)((uint32_t)src_pixel[0] * 0x3ff / 0xff);
         pixel |= ((uint32_t)((uint32_t)src_pixel[1] * 0x3ff / 0xff) << 10);
         pixel |= ((uint32_t)((uint32_t)src_pixel[2] * 0x3ff / 0xff) << 20);
         pixel |= ((uint32_t)(src_pixel[3] >> 6) << 30);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_l8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_a8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_i8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_l8a8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[2];
         *dst_pixel++ = src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_l16_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[2] * 0xffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z16_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[0] * 0xffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z32_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[0] * 0xffffffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z32_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (float)(src_pixel[0] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z24s8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= (uint32_t)((uint32_t)src_pixel[0] * 0xffffff / 0xff);
         pixel |= (src_pixel[1] << 24);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_s8z24_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= src_pixel[1];
         pixel |= ((uint32_t)((uint32_t)src_pixel[0] * 0xffffff / 0xff) << 8);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_z24x8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= (uint32_t)((uint32_t)src_pixel[0] * 0xffffff / 0xff);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_x8z24_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         uint32_t pixel = 0;
         pixel |= ((uint32_t)((uint32_t)src_pixel[0] * 0xffffff / 0xff) << 8);
         *dst_pixel++ = pixel;
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*8);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)(src_pixel[0] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64g64_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*16);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)(src_pixel[0] * (1.0f/0xff));
         *dst_pixel++ = (double)(src_pixel[1] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64g64b64_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*24);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)(src_pixel[0] * (1.0f/0xff));
         *dst_pixel++ = (double)(src_pixel[1] * (1.0f/0xff));
         *dst_pixel++ = (double)(src_pixel[2] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r64g64b64a64_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      double *dst_pixel = (double *)(dst_row + x0*32);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (double)(src_pixel[0] * (1.0f/0xff));
         *dst_pixel++ = (double)(src_pixel[1] * (1.0f/0xff));
         *dst_pixel++ = (double)(src_pixel[2] * (1.0f/0xff));
         *dst_pixel++ = (double)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (float)(src_pixel[0] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*8);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (float)(src_pixel[0] * (1.0f/0xff));
         *dst_pixel++ = (float)(src_pixel[1] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*12);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (float)(src_pixel[0] * (1.0f/0xff));
         *dst_pixel++ = (float)(src_pixel[1] * (1.0f/0xff));
         *dst_pixel++ = (float)(src_pixel[2] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32a32_float_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      float *dst_pixel = (float *)(dst_row + x0*16);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (float)(src_pixel[0] * (1.0f/0xff));
         *dst_pixel++ = (float)(src_pixel[1] * (1.0f/0xff));
         *dst_pixel++ = (float)(src_pixel[2] * (1.0f/0xff));
         *dst_pixel++ = (float)(src_pixel[3] * (1.0f/0xff));
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[0] * 0xffffffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*8);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[0] * 0xffffffff / 0xff);
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[1] * 0xffffffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*12);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[0] * 0xffffffff / 0xff);
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[1] * 0xffffffff / 0xff);
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[2] * 0xffffffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32a32_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*16);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[0] * 0xffffffff / 0xff);
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[1] * 0xffffffff / 0xff);
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[2] * 0xffffffff / 0xff);
         *dst_pixel++ = (uint32_t)((uint64_t)src_pixel[3] * 0xffffffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(src_pixel[0] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*8);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint32_t)(src_pixel[1] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*12);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint32_t)(src_pixel[1] >> 8);
         *dst_pixel++ = (uint32_t)(src_pixel[2] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r32g32b32a32_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint32_t *dst_pixel = (uint32_t *)(dst_row + x0*16);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint32_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint32_t)(src_pixel[1] >> 8);
         *dst_pixel++ = (uint32_t)(src_pixel[2] >> 8);
         *dst_pixel++ = (uint32_t)(src_pixel[3] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[0] * 0xffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[0] * 0xffff / 0xff);
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[1] * 0xffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*6);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[0] * 0xffff / 0xff);
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[1] * 0xffff / 0xff);
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[2] * 0xffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16a16_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*8);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[0] * 0xffff / 0xff);
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[1] * 0xffff / 0xff);
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[2] * 0xffff / 0xff);
         *dst_pixel++ = (uint16_t)((uint32_t)src_pixel[3] * 0xffff / 0xff);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(src_pixel[0] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint16_t)(src_pixel[1] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*6);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint16_t)(src_pixel[1] >> 8);
         *dst_pixel++ = (uint16_t)(src_pixel[2] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r16g16b16a16_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint16_t *dst_pixel = (uint16_t *)(dst_row + x0*8);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint16_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint16_t)(src_pixel[1] >> 8);
         *dst_pixel++ = (uint16_t)(src_pixel[2] >> 8);
         *dst_pixel++ = (uint16_t)(src_pixel[3] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*3);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[2];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8a8_unorm_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = src_pixel[0];
         *dst_pixel++ = src_pixel[1];
         *dst_pixel++ = src_pixel[2];
         *dst_pixel++ = src_pixel[3];
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*1);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(src_pixel[0] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*2);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint8_t)(src_pixel[1] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*3);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint8_t)(src_pixel[1] >> 8);
         *dst_pixel++ = (uint8_t)(src_pixel[2] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

static void
util_format_r8g8b8a8_uscaled_write_4ub(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h)
{
   unsigned x, y;
   uint8_t *dst_row = dst + y0*dst_stride;
   const uint8_t *src_row = src;
   for (y = 0; y < h; ++y) {
      uint8_t *dst_pixel = (uint8_t *)(dst_row + x0*4);
      const uint8_t *src_pixel = src_row;
      for (x = 0; x < w; ++x) {
         *dst_pixel++ = (uint8_t)(src_pixel[0] >> 8);
         *dst_pixel++ = (uint8_t)(src_pixel[1] >> 8);
         *dst_pixel++ = (uint8_t)(src_pixel[2] >> 8);
         *dst_pixel++ = (uint8_t)(src_pixel[3] >> 8);
         src_pixel += 4;
      }
      dst_row += dst_stride;
      src_row += src_stride/sizeof(*src_row);
   }
}

void
util_format_write_4ub(enum pipe_format format, const uint8_t *src, unsigned src_stride, void *dst, unsigned dst_stride, unsigned x, unsigned y, unsigned w, unsigned h)
{
   void (*func)(const uint8_t *src, unsigned src_stride, uint8_t *dst, unsigned dst_stride, unsigned x0, unsigned y0, unsigned w, unsigned h);
   switch(format) {
   case PIPE_FORMAT_B8G8R8A8_UNORM:
      func = &util_format_b8g8r8a8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_B8G8R8X8_UNORM:
      func = &util_format_b8g8r8x8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_A8R8G8B8_UNORM:
      func = &util_format_a8r8g8b8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_X8R8G8B8_UNORM:
      func = &util_format_x8r8g8b8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_A8B8G8R8_UNORM:
      func = &util_format_a8b8g8r8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_X8B8G8R8_UNORM:
      func = &util_format_x8b8g8r8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_B5G5R5A1_UNORM:
      func = &util_format_b5g5r5a1_unorm_write_4ub;
      break;
   case PIPE_FORMAT_B4G4R4A4_UNORM:
      func = &util_format_b4g4r4a4_unorm_write_4ub;
      break;
   case PIPE_FORMAT_B5G6R5_UNORM:
      func = &util_format_b5g6r5_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R10G10B10A2_UNORM:
      func = &util_format_r10g10b10a2_unorm_write_4ub;
      break;
   case PIPE_FORMAT_L8_UNORM:
      func = &util_format_l8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_A8_UNORM:
      func = &util_format_a8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_I8_UNORM:
      func = &util_format_i8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_L8A8_UNORM:
      func = &util_format_l8a8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_L16_UNORM:
      func = &util_format_l16_unorm_write_4ub;
      break;
   case PIPE_FORMAT_Z16_UNORM:
      func = &util_format_z16_unorm_write_4ub;
      break;
   case PIPE_FORMAT_Z32_UNORM:
      func = &util_format_z32_unorm_write_4ub;
      break;
   case PIPE_FORMAT_Z32_FLOAT:
      func = &util_format_z32_float_write_4ub;
      break;
   case PIPE_FORMAT_Z24S8_UNORM:
      func = &util_format_z24s8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_S8Z24_UNORM:
      func = &util_format_s8z24_unorm_write_4ub;
      break;
   case PIPE_FORMAT_Z24X8_UNORM:
      func = &util_format_z24x8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_X8Z24_UNORM:
      func = &util_format_x8z24_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R64_FLOAT:
      func = &util_format_r64_float_write_4ub;
      break;
   case PIPE_FORMAT_R64G64_FLOAT:
      func = &util_format_r64g64_float_write_4ub;
      break;
   case PIPE_FORMAT_R64G64B64_FLOAT:
      func = &util_format_r64g64b64_float_write_4ub;
      break;
   case PIPE_FORMAT_R64G64B64A64_FLOAT:
      func = &util_format_r64g64b64a64_float_write_4ub;
      break;
   case PIPE_FORMAT_R32_FLOAT:
      func = &util_format_r32_float_write_4ub;
      break;
   case PIPE_FORMAT_R32G32_FLOAT:
      func = &util_format_r32g32_float_write_4ub;
      break;
   case PIPE_FORMAT_R32G32B32_FLOAT:
      func = &util_format_r32g32b32_float_write_4ub;
      break;
   case PIPE_FORMAT_R32G32B32A32_FLOAT:
      func = &util_format_r32g32b32a32_float_write_4ub;
      break;
   case PIPE_FORMAT_R32_UNORM:
      func = &util_format_r32_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R32G32_UNORM:
      func = &util_format_r32g32_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R32G32B32_UNORM:
      func = &util_format_r32g32b32_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R32G32B32A32_UNORM:
      func = &util_format_r32g32b32a32_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R32_USCALED:
      func = &util_format_r32_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R32G32_USCALED:
      func = &util_format_r32g32_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R32G32B32_USCALED:
      func = &util_format_r32g32b32_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R32G32B32A32_USCALED:
      func = &util_format_r32g32b32a32_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R16_UNORM:
      func = &util_format_r16_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R16G16_UNORM:
      func = &util_format_r16g16_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R16G16B16_UNORM:
      func = &util_format_r16g16b16_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R16G16B16A16_UNORM:
      func = &util_format_r16g16b16a16_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R16_USCALED:
      func = &util_format_r16_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R16G16_USCALED:
      func = &util_format_r16g16_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R16G16B16_USCALED:
      func = &util_format_r16g16b16_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R16G16B16A16_USCALED:
      func = &util_format_r16g16b16a16_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R8_UNORM:
      func = &util_format_r8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R8G8_UNORM:
      func = &util_format_r8g8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R8G8B8_UNORM:
      func = &util_format_r8g8b8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R8G8B8A8_UNORM:
      func = &util_format_r8g8b8a8_unorm_write_4ub;
      break;
   case PIPE_FORMAT_R8_USCALED:
      func = &util_format_r8_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R8G8_USCALED:
      func = &util_format_r8g8_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R8G8B8_USCALED:
      func = &util_format_r8g8b8_uscaled_write_4ub;
      break;
   case PIPE_FORMAT_R8G8B8A8_USCALED:
      func = &util_format_r8g8b8a8_uscaled_write_4ub;
      break;
   default:
      debug_printf("unsupported format\n");
      return;
   }
   func(src, src_stride, (uint8_t *)dst, dst_stride, x, y, w, h);
}

